"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExpressServerAdapter = void 0;
const HttpResponse_1 = require("../../entities/HttpResponse");
const Server_1 = require("../../entities/Server");
const express_1 = __importDefault(require("express"));
require("express-async-errors");
const multer_1 = __importDefault(require("multer"));
const HttpRequest_1 = require("../../entities/HttpRequest");
const cors_1 = __importDefault(require("cors"));
class ExpressServerAdapter extends Server_1.Server {
    constructor(config, corsOptions) {
        super(config);
        this.app = (0, express_1.default)();
        this.app.use((0, cors_1.default)(corsOptions));
        this.app.use(express_1.default.json());
        this.app.use(express_1.default.urlencoded({ extended: true }));
        this.multer = (0, multer_1.default)({
            dest: this.config.uploadDir,
        });
    }
    parseResponse(res, response) {
        res.status(response.statusCode);
        for (const [name, value] of Object.entries(response.headers)) {
            res.setHeader(name, value);
        }
        if (response.headers['content-type'] === 'application/json') {
            return res.json(response.body);
        }
        return res.send(response.body);
    }
    parseRequest(req) {
        const query = {};
        const headers = {};
        for (const [key, value] of Object.entries(req.query)) {
            query[key] = Array.isArray(value) ? value.map((el) => String(el)) : String(value);
        }
        for (const [name, value] of Object.entries(req.headers)) {
            headers[name] = String(value);
        }
        const request = new HttpRequest_1.HttpRequest({
            headers,
            query,
            params: req.params,
            url: req.url,
            path: req.path,
            body: req.body,
        });
        if (req.files && Array.isArray(req.files))
            request.files = req.files;
        return request;
    }
    on(method, path, middlewares) {
        this.app[method](path, this.multer.any(), async (req, res) => {
            const request = this.parseRequest(req);
            const response = new HttpResponse_1.HttpResponse();
            try {
                for (const middleware of middlewares) {
                    const handlerResponse = await middleware(request, response, (error) => {
                        if (error)
                            throw error;
                    });
                    if (handlerResponse && handlerResponse instanceof HttpResponse_1.HttpResponse)
                        return this.parseResponse(res, handlerResponse);
                }
            }
            catch (error) {
                if (error instanceof Error && this.onError)
                    return this.parseResponse(res, this.onError(request, response, error));
                throw error;
            }
        });
    }
    listen(port, callback) {
        this.app.listen(port, callback);
    }
}
exports.ExpressServerAdapter = ExpressServerAdapter;
